@@ -1,6 +1,6 @@
 // The MIT License (MIT)
 //
-// Copyright (c) 2019 A. Orlenko
+// Copyright (c) 2019-2020 A. Orlenko
 // Copyright (c) 2014 J.C. Moyer
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -53,7 +53,7 @@ typedef struct rs_item {
 #define RS_INT(name, val)                                                      \
   { TY_INT, name, .int_val = val }
 
-#if LUA_VERSION_NUM == 503
+#if LUA_VERSION_NUM >= 503
 #define TY_LUAINT 1
 #define RS_LUAINT(name, val)                                                   \
   { TY_LUAINT, name, .lua_int_val = val }
@@ -84,6 +84,8 @@ const char *rs_int_type(int width) {
     return "i32";
   case 8:
     return "i64";
+  case 16:
+    return "i128";
   }
 }
 
@@ -96,6 +98,8 @@ const char *rs_uint_type(int width) {
     return "u32";
   case 8:
     return "u64";
+  case 16:
+    return "u128";
   }
 }
 
@@ -139,7 +143,7 @@ int write_int_item(FILE *f, const char *name, int value) {
   return fprintf(f, "pub const %s: c_int = %d;\n", name, value);
 }
 
-#if LUA_VERSION_NUM == 503
+#if LUA_VERSION_NUM >= 503
 int write_lua_int_item(FILE *f, const char *name, LUA_INTEGER value) {
   return fprintf(f, "pub const %s: LUA_INTEGER = " LUA_INTEGER_FMT ";\n", name,
                  value);
@@ -152,7 +156,7 @@ int write_str_item(FILE *f, const char *name, const char *value) {
   char *buf = malloc(bufsz);
   int ret;
   escape(value, buf, bufsz);
-  ret = fprintf(f, "pub const %s: &'static str = \"%s\";\n", name, buf);
+  ret = fprintf(f, "pub const %s: &str = \"%s\";\n", name, buf);
   free(buf);
   return ret;
 }
@@ -171,7 +175,7 @@ int write_item(FILE *f, const rs_item *c) {
   switch (c->type) {
   case TY_INT:
     return write_int_item(f, c->name, c->int_val);
-#if LUA_VERSION_NUM == 503
+#if LUA_VERSION_NUM >= 503
   case TY_LUAINT:
     return write_lua_int_item(f, c->name, c->lua_int_val);
 #endif
@@ -251,7 +255,7 @@ int main(int argc, const char **argv) {
       // == lauxlib.h ==========================================================
 
       RS_COMMENT("lauxlib.h"),
-#if LUA_VERSION_NUM == 503
+#if LUA_VERSION_NUM >= 503
       RS_INT("LUAL_NUMSIZES", LUAL_NUMSIZES),
 #endif
 
@@ -263,15 +267,21 @@ int main(int argc, const char **argv) {
       RS_STR("LUA_IOLIBNAME", LUA_IOLIBNAME),
       RS_STR("LUA_OSLIBNAME", LUA_OSLIBNAME),
       RS_STR("LUA_STRLIBNAME", LUA_STRLIBNAME),
-#if LUA_VERSION_NUM == 503
+#ifdef LUA_UTF8LIBNAME
       RS_STR("LUA_UTF8LIBNAME", LUA_UTF8LIBNAME),
 #endif
-#if LUA_VERSION_NUM >= 502
+#ifdef LUA_BITLIBNAME
       RS_STR("LUA_BITLIBNAME", LUA_BITLIBNAME),
 #endif
       RS_STR("LUA_MATHLIBNAME", LUA_MATHLIBNAME),
       RS_STR("LUA_DBLIBNAME", LUA_DBLIBNAME),
       RS_STR("LUA_LOADLIBNAME", LUA_LOADLIBNAME),
+#ifdef LUA_JITLIBNAME
+      RS_STR("LUA_JITLIBNAME", LUA_JITLIBNAME),
+#endif
+#ifdef LUA_FFILIBNAME
+      RS_STR("LUA_FFILIBNAME", LUA_FFILIBNAME),
+#endif
   };
 
   if (!write_items(f, glue_entries)) {
